@@ -1,15 +1,17 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # This file is part of Xpra.
-# Copyright (C) 2014-2019 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2014-2024 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import os
 import sys
+from typing import Any
 
-#default implementation uses pycups
-from xpra.util import envbool, print_nested_dict
-from xpra.os_util import WIN32
+# default implementation uses pycups
+from xpra.platform import platform_import
+from xpra.util.str_fn import print_nested_dict, pver
+from xpra.util.env import envbool
 from xpra.log import Logger
 
 log = Logger("printing")
@@ -17,33 +19,41 @@ log = Logger("printing")
 RAW_MODE = envbool("XPRA_PRINTER_RAW", False)
 
 
-def get_printers():
+def get_printers() -> dict[str, Any]:
     return {}
 
-def get_printer_attributes(_name):
+
+def get_printer_attributes(_name: str) -> list:
     return []
 
-def get_default_printer():
-    return None
 
-def print_files(printer, filenames, title, options):
-    raise Exception("no print implementation available")
+def get_default_printer() -> str:
+    return ""
 
-def printing_finished(_printpid):
+
+def print_files(printer: str, filenames, title: str, options):
+    raise RuntimeError("no print implementation available")
+
+
+def printing_finished(_printpid) -> bool:
     return True
 
-def init_printing(printers_modified_callback=None):     #pylint: disable=unused-argument
-    pass
 
-def cleanup_printing():
-    pass
+def init_printing(printers_modified_callback=None) -> None:  # pylint: disable=unused-argument
+    """ overridden in platform code """
+
+
+def cleanup_printing() -> None:
+    """ overridden in platform code """
 
 
 DEFAULT_MIMETYPES = ["application/pdf", "application/postscript"]
 
-MIMETYPES = None
-def get_mimetypes():
-    global MIMETYPES, DEFAULT_MIMETYPES
+MIMETYPES: list[str] | None = None
+
+
+def get_mimetypes() -> list[str]:
+    global MIMETYPES
     if MIMETYPES is None:
         v = os.environ.get("XPRA_PRINTING_MIMETYPES", )
         if v is not None:
@@ -52,7 +62,7 @@ def get_mimetypes():
             MIMETYPES = DEFAULT_MIMETYPES
         if RAW_MODE:
             MIMETYPES.append("raw")
-        #make it easier to test different mimetypes:
+        # make it easier to test different mimetypes:
         PREFERRED_MIMETYPE = os.environ.get("XPRA_PRINTING_PREFERRED_MIMETYPE")
         if PREFERRED_MIMETYPE:
             if PREFERRED_MIMETYPE in MIMETYPES:
@@ -65,38 +75,19 @@ def get_mimetypes():
     return MIMETYPES
 
 
-def get_info():
+def get_info() -> dict[str, Any]:
     return default_get_info()
 
-def default_get_info():
+
+def default_get_info() -> dict[str, Any]:
     return {
-            "mimetypes" :   {
-                ""         : get_mimetypes(),
-                "default"  : DEFAULT_MIMETYPES,
-                }
-            }
+        "mimetypes": {
+            "": get_mimetypes(),
+            "default": DEFAULT_MIMETYPES,
+        }
+    }
 
 
-#default implementation uses pycups:
-from xpra.platform import platform_import
-if not WIN32:
-    #pycups is not available on win32
-    try:
-        from xpra.platform.pycups_printing import (
-            get_printers,
-            print_files,
-            printing_finished,
-            init_printing,
-            cleanup_printing,
-            get_info,
-            )
-        assert get_printers and print_files and printing_finished and init_printing, cleanup_printing
-    except Exception as e:
-        log("cannot load pycups", exc_info=True)
-        log.warn("Warning: printer forwarding disabled:")
-        log.warn(" %s", e)
-        del e
-
 platform_import(globals(), "printing", False,
                 "init_printing",
                 "cleanup_printing",
@@ -108,7 +99,8 @@ platform_import(globals(), "printing", False,
                 "DEFAULT_MIMETYPES")
 
 
-def main(argv=[]):
+def main(argv) -> int:
+    # pylint: disable=import-outside-toplevel
     if "-v" in argv or "--verbose" in argv:
         from xpra.log import add_debug_category, enable_debug_for
         add_debug_category("printing")
@@ -122,47 +114,51 @@ def main(argv=[]):
         except ValueError:
             pass
 
-    from xpra.util import nonl, pver
+    from xpra.util.str_fn import nonl
+
     def dump_dict(d):
         pk = None
         try:
-            for pk,pv in d.items():
+            for pk, pv in d.items():
                 try:
                     if isinstance(pv, bytes):
                         sv = pv.decode("utf8")
                     else:
                         sv = nonl(pver(pv))
-                except Exception as e:
+                except Exception:
                     sv = repr(pv)
-                print("        %s : %s" % (pk.ljust(32), sv))
+                print(f"        {pk:32} : {sv}")
         except Exception as e:
-            print("        error on %s: %s" % (pk, e))
-            print("        raw attributes: " % d)
+            print(f"        error on {pk}: {e}")
+            print(f"        raw attributes: {d}")
+
     def dump_info(d):
         print("System Configuration:")
         print_nested_dict(d)
+
     def dump_printers(d):
         for k in sorted(d.keys()):
             v = d[k]
             print("Printers:")
-            print("* %s" % k)
+            print(f"* {k}")
             dump_dict(v)
             attr = get_printer_attributes(k)
             if attr:
                 print(" attributes:")
                 for a in attr:
-                    print("        %s" % a)
+                    print(f"        {a}")
+
     from xpra.platform import program_context
     from xpra.log import enable_color
-    from xpra.util import csv
+    from xpra.util.str_fn import csv
     with program_context("Printing", "Printing"):
         enable_color()
         try:
             init_printing()
         except Exception as e:
             print("Error: initializing the printing tool")
-            print(" %s" % e)
-        if len(argv)<=1:
+            print(f" {e}")
+        if len(argv) <= 1:
             dump_printers(get_printers())
             print("")
             dump_info(get_info())
@@ -171,29 +167,29 @@ def main(argv=[]):
         if not printers:
             print("Cannot print: no printers found")
             return 1
-        if len(argv)==2:
+        if len(argv) == 2:
             filename = argv[1]
             if not os.path.exists(filename):
-                print("Cannot print file '%s': file does not exist" % filename)
+                print(f"Cannot print file {filename!r}: file does not exist")
                 return 1
-            printer = get_default_printer()     #pylint: disable=assignment-from-none
+            printer = get_default_printer()  # pylint: disable=assignment-from-none
             if not printer:
-                printer = printers.keys()[0]
-                if len(printers)>1:
-                    print("More than one printer found: %s", csv(printer.keys()))
-            print("Using printer '%s'" % printer)
+                printer = list(printers.keys())[0]
+                if len(printers) > 1:
+                    print("More than one printer found: " + csv(printer.keys()))
+            print(f"Using printer {printer!r}")
             filenames = [filename]
-        if len(argv)>2:
+        if len(argv) > 2:
             printer = argv[1]
             if printer not in printers:
-                print("Invalid printer '%s'" % printer)
+                print(f"Invalid printer {printer!r}")
                 return 1
             filenames = argv[2:]
             for filename in filenames:
                 if not os.path.exists(filename):
-                    print("File '%s' does not exist" % filename)
+                    print(f"File {filename!r} does not exist")
                     return 1
-        print("Printing: %s" % csv(filenames))
+        print("Printing: " + csv(filenames))
         print_files(printer, filenames, "Print Command", {})
     return 0
 
